import 'package:senior_platform_authentication/senior_platform_authentication.dart';

import '../../data/repositories/secure_storage_repository.dart';

class CheckStoredTokenUsecase implements BaseUsecase<bool, NoParams> {
  late final SecureStorageRepository _secureStorageRepository;
  late final GetUserUsecase _getUserUsecase;

  CheckStoredTokenUsecase({
    SecureStorageRepository? secureStorageRepository,
    required GetUserUsecase getUserUsecase,
  }) {
    _secureStorageRepository =
        secureStorageRepository ?? SecureStorageRepositoryImpl();
    _getUserUsecase = getUserUsecase;
  }

  @override
  Future<bool> call(NoParams params) async {
    try {
      final token = await _secureStorageRepository.readToken();
      final tokenExpirationDate =
          await _secureStorageRepository.readTokenExpirationDate();

      if (token == null) {
        return false;
      }

      if (token.accessToken.isEmpty) {
        return false;
      }

      if (token.refreshToken.isEmpty) {
        return false;
      }

      if (tokenExpirationDate == null) {
        return false;
      }

      if (tokenExpirationDate.isBefore(DateTime.now())) {
        return false;
      }

      // If user isn`t logged will throw an exception
      await _getUserUsecase.call(token.accessToken);

      return true;
    } catch (_) {
      return false;
    }
  }
}
