import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail/mocktail.dart';
import 'package:senior_platform_authentication_ui/senior_platform_authentication_ui.dart';
import 'package:senior_platform_authentication_ui/src/core/authentication/data/repositories/secure_storage_repository.dart';

import '../../../../../mocks/token_mock.dart';

class MockSecureStorageRepository extends Mock
    implements SecureStorageRepository {}

void main() {
  late final GetStoredTokenUsecase getStoredTokenUsecase;
  late final SecureStorageRepository repository;

  setUpAll(() {
    repository = MockSecureStorageRepository();
    getStoredTokenUsecase =
        GetStoredTokenUsecase(secureStorageRepository: repository);
  });

  group('getStoredTokenUsecase', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => repository.readToken(),
      ).thenAnswer((_) async {
        return tokenMock;
      });

      verifyNever(
        () => repository.readToken(),
      );

      // Act
      final result = await getStoredTokenUsecase(NoParams());

      // Assert
      expect(result, isA<Token?>());

      verify(
        () => repository.readToken(),
      ).called(1);
    });

    test('should throw exception when repository throws', () async {
      // Arrange
      when(
        () => repository.readToken(),
      ).thenThrow(Exception('oops'));

      verifyNever(
        () => repository.readToken(),
      );

      // Act
      final action = getStoredTokenUsecase(NoParams());

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => repository.readToken(),
      ).called(1);
    });
  });
}
